@@ -14,8 +14,16 @@
 (use-modules (nyacc lalr))
 (use-modules (nyacc export))
 (use-modules (ice-9 pretty-print))
-(write-lalr-tables cpp-mach "cpptab.scm.new")
-(write-lalr-actions cpp-mach "cppact.scm.new")
+
+(define (module-path filename)
+  (string-append "../../../../module/nyacc/lang/c99/" filename))
+
+(begin
+  (write-lalr-actions cpp-mach "cppact.scm.new")
+  (write-lalr-tables cpp-mach "cpptab.scm.new")
+  (move-if-changed "cppact.scm.new" (module-path "cppact.scm"))
+  (move-if-changed "cpptab.scm.new" (module-path "cpptab.scm"))
+  )
 
 (let ((dt '(("A" . "1") ("B" . "2")))
       (sx (with-input-from-string
