@@ -5,25 +5,26 @@ extern "C" {
 #include "inc.h"
 
 eval_t x;
-
 struct foo;
 
+int d = 0x123; /* d comment */
+float f = 0.0; 
+
+#define OFFSET(T,V) (((T*)0)->V)
+
 typedef struct {
   /* hello */
   eval_t x; /* comment */
   int x;
-  /* world */
 } xyz_t;
 
-int ex1_foo(ex1_t *mod, double t, double *x, int *y) {
-   int i = mod->xyz.i;
-   ++i;
-   return i+2;
+int foo(int y) {
+  double d;
+
+  d = 0.0;
 }
 
-int x; // this is x 
 /* this is lone comment */
-
 #ifdef __cplusplus__
 }
 #endif
