@@ -19,12 +19,20 @@
 (with-output-to-file "gram.y.new"
   (lambda () (lalr->bison js-spec)))
 (with-output-to-file "lang.txt.new"
-  (lambda () (pp-lalr-grammar js-spec)
-             (pp-lalr-machine js-mach)))
+  (lambda ()
+    (let* ((notice (assq-ref (assq-ref js-spec 'attr) 'notice))
+           (lines (if notice (string-split notice #\newline) '())))
+      (for-each (lambda (l) (simple-format #t "  ~A\n" l)) lines)
+      (newline))
+    (pp-lalr-grammar js-spec)
+    (pp-lalr-machine js-mach)))
 (write-lalr-actions js-mach "actions.scm.new")
 (write-lalr-tables js-mach "tables.scm.new")
 
 (define res (with-input-from-file "ex1.js" dev-parse-js))
 (pretty-print res)
 
-;; --- last line
+(use-modules (nyacc lang javascript pprint))
+;;(pretty-print-js res)
+
+;; --- last line ---
