@@ -33,7 +33,7 @@
    ;; Identifier => '$ident
    (lambda ($1 . $rest) `(Identifier ,$1))
    ;; PrimaryExpression => "this"
-   (lambda ($1 . $rest) `(PrimaryExpression ,$1))
+   (lambda ($1 . $rest) `(PrimaryExpression (this)))
    ;; PrimaryExpression => Identifier
    (lambda ($1 . $rest) `(PrimaryExpression ,$1))
    ;; PrimaryExpression => Literal
@@ -41,8 +41,7 @@
    ;; PrimaryExpression => ArrayLiteral
    (lambda ($1 . $rest) `(PrimaryExpression ,$1))
    ;; PrimaryExpression => "(" Expression ")"
-   (lambda ($3 $2 $1 . $rest)
-     `(PrimaryExpression ,$2))
+   (lambda ($3 $2 $1 . $rest) $2)
    ;; ArrayLiteral => "[" Elision "]"
    (lambda ($3 $2 $1 . $rest)
      `(ArrayLiteral (Elision ,(number->string $2))))
@@ -90,9 +89,9 @@
    ;; MemberExpression => PrimaryExpression
    (lambda ($1 . $rest) $1)
    ;; MemberExpression => MemberExpression "[" Expression "]"
-   (lambda ($4 $3 $2 $1 . $rest) `(ary-ref ,$1 ,$3))
+   (lambda ($4 $3 $2 $1 . $rest) `(ary-ref ,$3 ,$1))
    ;; MemberExpression => MemberExpression "." Identifier
-   (lambda ($3 $2 $1 . $rest) `(obj-ref ,$1 ,$3))
+   (lambda ($3 $2 $1 . $rest) `(obj-ref ,$3 ,$1))
    ;; MemberExpression => "new" MemberExpression Arguments
    (lambda ($3 $2 $1 . $rest) `(new ,$2 ,$3))
    ;; NewExpression => MemberExpression
@@ -106,13 +105,14 @@
    (lambda ($2 $1 . $rest)
      `(CallExpression ,$1 ,$2))
    ;; CallExpression => CallExpression "[" Expression "]"
-   (lambda ($4 $3 $2 $1 . $rest) `(ary-ref ,$1 ,$3))
+   (lambda ($4 $3 $2 $1 . $rest) `(ary-ref ,$3 ,$1))
    ;; CallExpression => CallExpression "." Identifier
-   (lambda ($3 $2 $1 . $rest) `(obj-ref ,$1 ,$3))
+   (lambda ($3 $2 $1 . $rest) `(obj-ref ,$3 ,$1))
    ;; Arguments => "(" ")"
-   (lambda ($2 $1 . $rest) '(ArgumentList))
+   (lambda ($2 $1 . $rest) '(Arguments))
    ;; Arguments => "(" ArgumentList ")"
-   (lambda ($3 $2 $1 . $rest) (tl->list $2))
+   (lambda ($3 $2 $1 . $rest)
+     `(Arguments ,(tl->list $2)))
    ;; ArgumentList => AssignmentExpression
    (lambda ($1 . $rest) (make-tl 'ArgumentList $1))
    ;; ArgumentList => ArgumentList "," AssignmentExpression
@@ -142,7 +142,7 @@
    ;; UnaryExpression => "++" UnaryExpression
    (lambda ($2 $1 . $rest) `(pre-inc ,$2))
    ;; UnaryExpression => "--" UnaryExpression
-   (lambda ($2 $1 . $rest) `(pre-inc ,$2))
+   (lambda ($2 $1 . $rest) `(pre-dec ,$2))
    ;; UnaryExpression => "+" UnaryExpression
    (lambda ($2 $1 . $rest) `(pos ,$2))
    ;; UnaryExpression => "-" UnaryExpression
@@ -188,14 +188,13 @@
    ;; EqualityExpression => RelationalExpression
    (lambda ($1 . $rest) $1)
    ;; EqualityExpression => EqualityExpression "==" RelationalExpression
-   (lambda ($3 $2 $1 . $rest) `(equal ,$1 ,$3))
+   (lambda ($3 $2 $1 . $rest) `(eq ,$1 ,$3))
    ;; EqualityExpression => EqualityExpression "!=" RelationalExpression
-   (lambda ($3 $2 $1 . $rest) `(not-equal ,$1 ,$3))
+   (lambda ($3 $2 $1 . $rest) `(neq ,$1 ,$3))
    ;; EqualityExpression => EqualityExpression "===" RelationalExpression
-   (lambda ($3 $2 $1 . $rest) `(equal-eq ,$1 ,$3))
+   (lambda ($3 $2 $1 . $rest) `(eq-eq ,$1 ,$3))
    ;; EqualityExpression => EqualityExpression "!==" RelationalExpression
-   (lambda ($3 $2 $1 . $rest)
-     `(not-equal-eq ,$1 ,$3))
+   (lambda ($3 $2 $1 . $rest) `(neq-eq ,$1 ,$3))
    ;; BitwiseANDExpression => EqualityExpression
    (lambda ($1 . $rest) $1)
    ;; BitwiseANDExpression => BitwiseANDExpression "&" EqualityExpression
@@ -289,9 +288,9 @@
    ;; Statement => TryStatement
    (lambda ($1 . $rest) $1)
    ;; Block => "{" StatementList "}"
-   (lambda ($3 $2 $1 . $rest) $1)
+   (lambda ($3 $2 $1 . $rest) `(Block ,$2))
    ;; Block => "{" "}"
-   (lambda ($2 $1 . $rest) $1)
+   (lambda ($2 $1 . $rest) '(Block))
    ;; StatementList => Statement
    (lambda ($1 . $rest) (make-tl 'StatementList $1))
    ;; StatementList => StatementList Statement
@@ -318,7 +317,8 @@
    ;; EmptyStatement => ";"
    (lambda ($1 . $rest) '(EmptyStatement))
    ;; ExpressionStatement => Expression ";"
-   (lambda ($2 $1 . $rest) $1)
+   (lambda ($2 $1 . $rest)
+     `(ExpressionStatement ,$1))
    ;; IfStatement => "if" "(" Expression ")" Statement "else" Statement
    (lambda ($7 $6 $5 $4 $3 $2 $1 . $rest)
      `(IfStatement ,$3 ,$5 ,$7))
