@@ -25,6 +25,7 @@
 	    make-lalr-machine
 	    compact-machine hashify-machine
 	    lalr-match-table
+	    restart-spec
 	    pp-lalr-grammar pp-lalr-machine
 	    write-lalr-tables
 	    write-lalr-actions
@@ -40,7 +41,7 @@
   #:use-module ((nyacc parse) #:select (wrap-action))
   )
 
-(define *nyacc-version* "0.64.0")
+(define *nyacc-version* "0.65.0")
 
 ;; @item proxy-? sym rhs
 ;; @example
@@ -458,7 +459,6 @@
 	       (cons 'non-terms nl)
 	       (cons 'lhs-v (list->vector (reverse ll)))
 	       (cons 'rhs-v (map-attr->vector al 'rhs))
-	       ;;(cons 'prune-al TODO: filter 'with from '@l)
 	       (cons 'terminals tl)
 	       (cons 'start start-symbol)
 	       (cons 'attr (list (cons 'expect (or (assq-ref tree 'expect) 0))
@@ -483,32 +483,33 @@
 ;; This record holds the minimum data from the grammar needed to build the
 ;; machine from the grammar specification.
 (define-record-type lalr-core-type
-  (make-lalr-core non-terms terminals lhs-v rhs-v prune-al eps-l)
+  (make-lalr-core non-terms terminals start lhs-v rhs-v eps-l)
   lalr-core-type?
   (non-terms core-non-terms)	      ; list of non-terminals
   (terminals core-terminals)	      ; list of non-terminals
+  (start core-start)		      ; start non-terminal
   (lhs-v core-lhs-v)		      ; vec of left hand sides
   (rhs-v core-rhs-v)		      ; vec of right hand sides
-  (prune-al core-prune-al)	      ; a-list of prunage
   (eps-l core-eps-l))		      ; non-terms w/ eps prod's
 
 ;; @item make-core spec => lalr-core-type
 (define (make-core spec)
   (make-lalr-core (assq-ref spec 'non-terms)
 		  (assq-ref spec 'terminals)
+		  (assq-ref spec 'start)
 		  (assq-ref spec 'lhs-v)
 		  (assq-ref spec 'rhs-v)
-		  '() '()))
+		  '()))
 
 ;; @item make-core/extras spec => lalr-core-type
 ;; Add list of symbols with epsilon productions.
 (define (make-core/extras spec)
   (let ((non-terms (assq-ref spec 'non-terms))
 	(terminals (assq-ref spec 'terminals))
+	(start (assq-ref spec 'start))
 	(lhs-v (assq-ref spec 'lhs-v))
 	(rhs-v (assq-ref spec 'rhs-v)))
-    (make-lalr-core non-terms terminals lhs-v rhs-v
-		    (assq-ref spec 'prune-al)
+    (make-lalr-core non-terms terminals start lhs-v rhs-v
 		    (find-eps non-terms lhs-v rhs-v))))
 
 
@@ -609,6 +610,14 @@
     (if (last-item? item)
 	'$epsilon
 	(vector-ref rule (cdr item)))))
+(define (NEW-looking-at item)
+  ;; If (0 . 0) use start symbol.
+  (let* ((core (fluid-ref *lalr-core*)) (gx (car item)))
+    (if (last-item? item)
+	'$epsilon
+	(if (zero? gx)
+	    (core-start core)
+	    (vector-ref (vector-ref (core-rhs-v core) gx) (cdr item))))))
 
 ;; @item first-item gx
 ;; Given grammar rule index return the first item.
@@ -695,87 +704,6 @@
        (else
 	;; Done, so return.
 	(reverse rslt))))))
-;; To support pruning:
-;; Maybe replace each symbol in next, done with a list, reflecting derivations.
-;; That is,
-;; 1: A  -> B | C
-;; 2: B  -> P1
-;; 3: P1 -> E, prune F
-;; 4: B  -> P2
-;; 5: P2 -> E, prune G
-;; 6: E  -> F
-;; 7: E  -> G
-;; 8: C  -> ";"
-;;
-;; curr=A next=((B A) (C A)) done=(A) rslt=(1)
-;; OR
-;; use alist of LHS to add, but prune xxx
-;; curr=((A)) next=(B C) done=(A) rslt=(1)
-;; curr=((B) (C)) next=(P1 P2) done=(A B C) rslt=(1 2 4)
-;; curr=((P1 F) (P2 G)) next=((E F) (E G))
-;;     done=(A B C P1 P2) rslt=(1 2 4 3 5)
-;; curr=((P1 F) (P2 G)) next=((E F) (E G) => (E (intersection F G))
-;; curr=((E)) next=((E F) (E G) => (E (intersection F G))
-
-;; @item p-todo symbol prunage spl
-;; updates to be handled
-;; if want stuff in spl not in E
-;; @example
-;; p-memb 'E '(F) '((E G)) => 
-;; @end example
-(define (p-todo symbol prunage spl)
-  (let* ((pnag (assq-ref symbol spl))
-	 (pset (and=> pnag (lambda (p) (lset-intersection p prunage))))
-	 )
-    (if (and pnag pset)
-	#f
-	#t)))
-(define (p-cons symbol prunage spl)
-  #f)
-(define (with-gram gram thunk)
-  (fluid-set! *lalr-core* (make-core/extras gram))
-  (thunk))
-(define (non-kernels/prunage symb)
-  (let* ((core (fluid-ref *lalr-core*))
-	 (lhs-v (core-lhs-v core))
-	 (prune (core-prune-al core))
-	 (glen (vector-length lhs-v))
-	 (lhs-symb (lambda (gx) (vector-ref lhs-v gx)))
-	 (inters lset-intersection)
-	 )
-    (let iter ((rslt '())		; result is set of p-rule indices
-	       (done '())		; symbols completed or queued
-	       (next '())		; next round of symbols to process
-	       (curr (list (list symb))) ; this round of symbols to process
-	       (gx 0))			; p-rule index
-      (cond
-       ((< gx glen)
-	(if (assq (lhs-symb gx) curr)
-	    (let* ((pair (assq (lhs-symb gx) curr))
-		   (rhs1 (looking-at (first-item gx))) ; 1st-RHS-sym | $eps
-		   (pnge (assq-ref prune rhs1))	      ; prunage, AA null
-		   (rhsx (assq rhs1 prune))
-		   ;;
-		   (rslt1 (if (memq gx rslt) rslt (cons gc rslt)))
-		   (done1 (if (assq rhs1 done)
-			      (cons 
-			       (cons rhs1 (inters (assq-ref done rhs1) rhsx))
-			       done)
-			      done))
-		   (next1 (cond ((memq rhs1 done) next)
-				((terminal? rhs1) next)
-				(else (cons rhs1 next))))
-		   
-		   )
-	      (iter rslt1 done1 next1 curr (1+ gx)))
-	    ;; Nothing to check; process next rule.
-	    (iter rslt done next curr (1+ gx))))
-       ((pair? next)
-	;; Start another sweep throught the grammar.
-	(iter rslt done '() next 0))
-       (else
-	;; Done, so return.
-	(reverse rslt))))))
 
 ;; @item expand-k-item => item-set
 ;; Expand a kernel-item into a list with the non-kernels.
@@ -934,9 +862,7 @@
 ;; result will include @code{end-token-list}.
 (define (first symbol-list end-token-list)
   (let* ((core (fluid-ref *lalr-core*))
-	 (eps-l (core-eps-l core))
-	 (prune-al (core-prune-al core)) ; alist: lhs -> prune-l
-	 )
+	 (eps-l (core-eps-l core)))
     ;; This loop strips off the leading symbol from stng and then adds to
     ;; todo list, which results in range of p-rules getting checked for
     ;; terminals.
@@ -968,41 +894,29 @@
 	(iter rslt pngl stng hzeps done todo
 	      (range-next p-range) (first-item (car p-range))))
 
-      ((pair? todo)
-       ;; Process the next non-terminal for grammar rules, unless it
-       ;; is in the prungage list.  And if it has prunage, add that.
-       (cond
-	((memq (car todo) pngl)		; on prunage-list, skip
-	 ;;(fmtout "skipping ~S\n" (car todo))
-	 (iter rslt pngl stng hzeps done (cdr todo) '() '()))
-	((assq-ref prune-al (car todo)) => ; has prunage, add, and go
-	 (lambda (l)
-	   (iter rslt (append l pngl) stng hzeps done
-	       (cdr todo) (prule-range (car todo)) '())))
-	(else				; just go
-	 (iter rslt pngl stng hzeps done
-	       (cdr todo) (prule-range (car todo)) '())))
-       )
+       ((pair? todo)
+	(iter rslt pngl stng hzeps done
+	      (cdr todo) (prule-range (car todo)) '()))
 
-      ((and hzeps (pair? stng))
-       ;; Last pass saw an $epsilon so check the next input symbol,
-       ;; with saweps reset to #f.
-       (let* ((symb (car stng)) (stng1 (cdr stng)) (symbl (list symb)))
-	 (if (terminal? symb)
-	     (iter (cons symb rslt) pngl stng1
-		   (and hzeps (memq symb eps-l))
-		   done todo p-range '())
-	     (iter rslt pngl stng1
-		   (or (eq? symb '$epsilon) (memq symb eps-l))
-		   symbl symbl '() '()))))
-      (hzeps
-       ;; $epsilon passes all the way through.
-       ;; If end-token-list provided use that.
-       (if (pair? end-token-list)
-	   (lset-union eqv? rslt end-token-list)
-	   (cons '$epsilon rslt)))
-      (else
-       rslt)))))
+       ((and hzeps (pair? stng))
+	;; Last pass saw an $epsilon so check the next input symbol,
+	;; with saweps reset to #f.
+	(let* ((symb (car stng)) (stng1 (cdr stng)) (symbl (list symb)))
+	  (if (terminal? symb)
+	      (iter (cons symb rslt) pngl stng1
+		    (and hzeps (memq symb eps-l))
+		    done todo p-range '())
+	      (iter rslt pngl stng1
+		    (or (eq? symb '$epsilon) (memq symb eps-l))
+		    symbl symbl '() '()))))
+       (hzeps
+	;; $epsilon passes all the way through.
+	;; If end-token-list provided use that.
+	(if (pair? end-token-list)
+	    (lset-union eqv? rslt end-token-list)
+	    (cons '$epsilon rslt)))
+       (else
+	rslt)))))
 
 ;; @item item->stng item => list-of-symbols
 ;; Convert item (e.g., @code{(1 . 2)}) to list of symbols to the end of the
@@ -1584,6 +1498,18 @@
 ;;   rhs-v - right hand sides
 ;;   pat-v - action table
 
+;; @item restart-spec spec start => spec
+;; This generates a new spec with a different start.
+;; @example
+;; (restart-spec clang-spec 'expression) => cexpr-spec
+;; @end example
+(define (restart-spec spec start)
+  (let* ((rhs-v (vector-copy (assq-ref spec 'rhs-v))))
+    (vector-set! rhs-v 0 (vector start))
+    (cons* (cons 'start start)
+	   (cons 'rhs-v rhs-v)
+	   spec)))
+
 ;; @item make-lalr-machine spec => pgen
 ;; Generate a-list of items used for building/debugging parsers.
 ;; It might be useful to add hashify and compact with keyword arguments.
@@ -1623,6 +1549,21 @@
     (fmt port "~A =>" lhs)
     (vector-for-each (lambda (ix e) (fmt port " ~A" (elt->str e tl))) rhs)
     (newline port)))
+(define (NEW-pp-rule il gx . rest)
+  ;; This uses start for rule 0.
+  (let* ((port (if (pair? rest) (car rest) (current-output-port)))
+	 (core (fluid-ref *lalr-core*))
+	 (lhs (vector-ref (core-lhs-v core) gx))
+	 (rhs (vector-ref (core-rhs-v core) gx))
+	 (tl (core-terminals core)))
+    (cond
+     ((zero? gx)
+      (fmt port "$start => ~A\n" (core-start core)))
+     (else
+      (display (substring "                     " 0 (min il 20)) port)
+      (fmt port "~A =>" lhs)
+      (vector-for-each (lambda (ix e) (fmt port " ~A" (elt->str e tl))) rhs)
+      (newline port)))))
 	 
 ;; @item pp-item item => string
 ;; This could be called item->string.
@@ -1644,10 +1585,32 @@
 		  sl (if (= rx (cdr item)) '(" .") '())
 		  (let ((e (vector-ref rhs rx)))
 		    (list (string-append " " (elt->str e tl)))))))))))
+(define (NEW-pp-item item)
+  (let* ((core (fluid-ref *lalr-core*))
+	 (tl (core-terminals core))
+	 (gx (car item))
+	 (lhs (vector-ref (core-lhs-v core) gx))
+	 (rhs (vector-ref (core-rhs-v core) gx))
+	 (rhs-len (vector-length rhs)))
+    (cond
+     ((zero? gx)
+      (if (zero? (cdr item))
+	  (fmt #f "$start => . ~A" (core-start core))
+	  (fmt #f "$start => ~A ." (core-start core))))
+     (else
+      (apply
+       string-append
+       (let iter ((rx 0) (sl (list (fmtstr "~S =>" lhs))))
+	 (if (= rx rhs-len)
+	     (append sl (if (= -1 (cdr item)) '(" .") '()))
+	     (iter (1+ rx)
+		   (append
+		    sl (if (= rx (cdr item)) '(" .") '())
+		    (let ((e (vector-ref rhs rx)))
+		      (list (string-append " " (elt->str e tl)))))))))))))
 
 ;; @item pp-lalr-grammar spec [port]
 ;; Pretty-print the grammar to the specified port, or current output.
-;; TODO: Add @code{$prec} and @code{$prune} expressions.
 (define (pp-lalr-grammar spec . rest)
   (let* ((port (if (pair? rest) (car rest) (current-output-port)))
 	 (lhs-v (assq-ref spec 'lhs-v))
@@ -1884,33 +1847,6 @@
     (ugly-print (assq-ref mach name) port)
     (fmt port ")\n\n"))
 
-  (define (pp-rule/ts gx)
-    (let* ((core (fluid-ref *lalr-core*))
-	   (lhs (vector-ref (core-lhs-v core) gx))
-	   (rhs (vector-ref (core-rhs-v core) gx))
-	   (tl (core-terminals core))
-	   (line (string-append
-		  (symbol->string lhs) " => "
-		  (string-join 
-		   (map (lambda (elt) (elt->str elt tl))
-			(vector->list rhs))
-		   " "))))
-      (if (> (string-length line) 72)
-	  (string-append (substring/shared line 0 69) "...")
-	  line)))
-    
-  (define (write-actions mach port)
-    (with-fluid*
-     *lalr-core* (make-core mach)
-     (lambda ()
-       (fmt port "(define act-v\n  (vector\n")
-       (vector-for-each
-	(lambda (ix actn)
-	  (fmt port "   ;; ~A\n" (pp-rule/ts ix))
-	  (pretty-print (wrap-action actn) port #:per-line-prefix "   "))
-	(assq-ref mach 'act-v))
-       (fmt port "   ))\n\n"))))
-
   (call-with-output-file filename
     (lambda (port)
       (fmt port ";; ~A\n\n"
@@ -1920,11 +1856,18 @@
       (write-table mach 'pat-v port)
       (write-table mach 'rto-v port)
       (write-table mach 'mtab port)
-      ;;(write-actions mach port)
       (display ";;; end tables" port)
       (newline port))))
 
+
+;; @item write-lalr-actions mach filename [#:lang output-lang]
+;; For example,
+;; @example
+;; write-lalr-actions mach "actions.scm"
+;; write-lalr-actions mach "actions.tcl" #:lang 'tcl
+;; @end example
 (define* (write-lalr-actions mach filename #:key (lang 'scheme))
+
   (define (pp-rule/ts gx)
     (let* ((core (fluid-ref *lalr-core*))
 	   (lhs (vector-ref (core-lhs-v core) gx))
@@ -1939,6 +1882,21 @@
       (if (> (string-length line) 72)
 	  (string-append (substring/shared line 0 69) "...")
 	  line)))
+  (define (NEW-pp-rule/ts gx)
+    ;; TBD: use start for zeroth rule
+    (let* ((core (fluid-ref *lalr-core*))
+	   (lhs (vector-ref (core-lhs-v core) gx))
+	   (rhs (vector-ref (core-rhs-v core) gx))
+	   (tl (core-terminals core))
+	   (line (string-append
+		  (symbol->string lhs) " => "
+		  (string-join 
+		   (map (lambda (elt) (elt->str elt tl))
+			(vector->list rhs))
+		   " "))))
+      (if (> (string-length line) 72)
+	  (string-append (substring/shared line 0 69) "...")
+	  line)))
     
   (define (write-actions mach port)
     (with-fluid*
@@ -1946,8 +1904,8 @@
      (lambda ()
        (fmt port "(define act-v\n  (vector\n")
        (vector-for-each
-	(lambda (ix actn)
-	  (fmt port "   ;; ~A\n" (pp-rule/ts ix))
+	(lambda (gx actn)
+	  (fmt port "   ;; ~A\n" (pp-rule/ts gx))
 	  (pretty-print (wrap-action actn) port #:per-line-prefix "   "))
 	(assq-ref mach 'act-v))
        (fmt port "   ))\n\n"))))
